from typing import Dict, Optional

from pydantic import BaseModel, ConfigDict

from ..models.currency import Currency


class InvoiceLineItem(BaseModel):
    """An invoice line item."""

    amount: float = 0.0

    currency: Currency = "usd"  # type: ignore

    description: Optional[str] = None

    id: Optional[str] = None

    invoice_item: Optional[str] = None

    metadata: Dict[str, str] = {}

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class ApiCallStatus(str, Enum):
    """The status of an async API call."""  # noqa: E501

    """# The async API call is queued. """  # noqa: E501
    QUEUED = "queued"
    """# The async API call was uploaded to be converted. """  # noqa: E501
    UPLOADED = "uploaded"
    """# The async API call is in progress. """  # noqa: E501
    IN_PROGRESS = "in_progress"
    """# The async API call has completed. """  # noqa: E501
    COMPLETED = "completed"
    """# The async API call has failed. """  # noqa: E501
    FAILED = "failed"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.unit_mass import UnitMass


class Mass(BaseModel):
    """The mass response."""

    mass: float

    output_unit: UnitMass

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.coupon import Coupon


class Discount(BaseModel):
    """The resource representing a Discount."""

    coupon: Coupon

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class AppClientInfo(BaseModel):
    """Information about a third party app client."""

    url: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class ModelingAppSubscriptionTierName(str, Enum):
    """An enum representing a Modeling App subscription tier name."""  # noqa: E501

    """# The free tier. """  # noqa: E501
    FREE = "free"
    """# The pro tier. """  # noqa: E501
    PRO = "pro"
    """# The team tier. """  # noqa: E501
    TEAM = "team"
    """# The enterprise tier. """  # noqa: E501
    ENTERPRISE = "enterprise"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitForce(str, Enum):
    """The valid types of force units."""  # noqa: E501

    """# Dynes <https://en.wikipedia.org/wiki/Dyne> """  # noqa: E501
    DYNES = "dynes"
    """# Kiloponds <https://en.wikipedia.org/wiki/Kilopond> """  # noqa: E501
    KILOPONDS = "kiloponds"
    """# Micronewtons <https://en.wikipedia.org/wiki/Newton_(unit)> """  # noqa: E501
    MICRONEWTONS = "micronewtons"
    """# Millinewtons <https://en.wikipedia.org/wiki/Newton_(unit)> """  # noqa: E501
    MILLINEWTONS = "millinewtons"
    """# Newtons <https://en.wikipedia.org/wiki/Newton_(unit)> """  # noqa: E501
    NEWTONS = "newtons"
    """# Poundals <https://en.wikipedia.org/wiki/Poundal> """  # noqa: E501
    POUNDALS = "poundals"
    """# Pounds <https://en.wikipedia.org/wiki/Pound_(force)> """  # noqa: E501
    POUNDS = "pounds"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.angle import Angle
from ..models.origin_type import OriginType
from ..models.point3d import Point3d


class Rotation(BaseModel):
    """A rotation defined by an axis, origin of rotation, and an angle."""

    angle: Angle

    axis: Point3d

    origin: OriginType

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.axis import Axis
from ..models.direction import Direction


class AxisDirectionPair(BaseModel):
    """An [`Axis`] paired with a [`Direction`]."""

    axis: Axis

    direction: Direction

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict


class KclCodeCompletionResponse(BaseModel):
    """A response with KCL code completions."""

    completions: List[str]

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.address_details import AddressDetails


class BillingInfo(BaseModel):
    """The billing information for payments."""

    address: Optional[AddressDetails] = None

    name: Optional[str] = None

    phone: str = ""

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict


class Solid3dGetAllOppositeEdges(BaseModel):
    """The response from the `Solid3dGetAllOppositeEdges` command."""

    edges: List[str]

    model_config = ConfigDict(protected_namespaces=())



import datetime
from typing import Dict

from pydantic import BaseModel, ConfigDict

from ..models.cluster import Cluster
from ..models.gateway import Gateway
from ..models.jetstream import Jetstream
from ..models.leaf_node import LeafNode


class Connection(BaseModel):
    """Metadata about a pub-sub connection.

    This is mostly used for internal purposes and debugging."""

    auth_timeout: int = 0

    cluster: Cluster = {
        "addr": None,
        "auth_timeout": 0,
        "cluster_port": 0,
        "name": "",
        "tls_timeout": 0,
        "urls": [],
    }  # type: ignore

    config_load_time: datetime.datetime

    connections: int = 0

    cores: int = 0

    cpu: float = 0.0

    gateway: Gateway = {
        "auth_timeout": 0,
        "host": "",
        "name": "",
        "port": 0,
        "tls_timeout": 0,
    }  # type: ignore

    git_commit: str = ""

    go: str = ""

    gomaxprocs: int = 0

    host: str

    http_base_path: str = ""

    http_host: str = ""

    http_port: int = 0

    http_req_stats: Dict[str, int]

    https_port: int = 0

    in_bytes: int = 0

    in_msgs: int = 0

    jetstream: Jetstream = {
        "config": {"domain": "", "max_memory": 0, "max_storage": 0, "store_dir": ""},
        "meta": {"cluster_size": 0, "leader": "", "name": ""},
        "stats": {
            "accounts": 0,
            "api": {"errors": 0, "inflight": 0, "total": 0},
            "ha_assets": 0,
            "memory": 0,
            "reserved_memory": 0,
            "reserved_store": 0,
            "store": 0,
        },
    }  # type: ignore

    leaf: LeafNode = {"auth_timeout": 0, "host": "", "port": 0, "tls_timeout": 0}  # type: ignore

    leafnodes: int = 0

    max_connections: int = 0

    max_control_line: int = 0

    max_payload: int = 0

    max_pending: int = 0

    mem: int = 0

    now: datetime.datetime

    out_bytes: int = 0

    out_msgs: int = 0

    ping_interval: int = 0

    ping_max: int = 0

    port: int = 0

    proto: int = 0

    remotes: int = 0

    routes: int = 0

    server_id: str = ""

    server_name: str = ""

    slow_consumers: int = 0

    start: datetime.datetime

    subscriptions: int = 0

    system_account: str = ""

    tls_timeout: int = 0

    total_connections: int = 0

    uptime: str = ""

    version: str = ""

    write_deadline: int = 0

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class CameraDragInteractionType(str, Enum):
    """The type of camera drag interaction."""  # noqa: E501

    """# Camera pan """  # noqa: E501
    PAN = "pan"
    """# Camera rotate (revolve/orbit) """  # noqa: E501
    ROTATE = "rotate"
    """# Camera zoom (increase or decrease distance to reference point center) """  # noqa: E501
    ZOOM = "zoom"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PlyStorage(str, Enum):
    """The storage for the output PLY file."""  # noqa: E501

    """# Write numbers in their ascii representation (e.g. -13, 6.28, etc.). Properties are separated by spaces and elements are separated by line breaks. """  # noqa: E501
    ASCII = "ascii"
    """# Encode payload as binary using little endian. """  # noqa: E501
    BINARY_LITTLE_ENDIAN = "binary_little_endian"
    """# Encode payload as binary using big endian. """  # noqa: E501
    BINARY_BIG_ENDIAN = "binary_big_endian"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Dict, Optional

from pydantic import BaseModel, ConfigDict

from ..models.billing_info import BillingInfo
from ..models.card_details import CardDetails
from ..models.payment_method_type import PaymentMethodType


class PaymentMethod(BaseModel):
    """A payment method."""

    billing_info: BillingInfo

    card: Optional[CardDetails] = None

    created_at: datetime.datetime

    id: Optional[str] = None

    metadata: Dict[str, str] = {}

    type: PaymentMethodType

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict

from ..models.extrusion_face_info import ExtrusionFaceInfo


class Solid3dGetExtrusionFaceInfo(BaseModel):
    """Extrusion face info struct (useful for maintaining mappings between source path segment ids and extrusion faces)"""

    faces: List[ExtrusionFaceInfo]

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class ClientMetrics(BaseModel):
    """ClientMetrics contains information regarding the state of the peer."""

    rtc_frames_decoded: int

    rtc_frames_dropped: int

    rtc_frames_per_second: int

    rtc_frames_received: int

    rtc_freeze_count: int

    rtc_jitter_sec: float

    rtc_keyframes_decoded: int

    rtc_total_freezes_duration_sec: float

    model_config = ConfigDict(protected_namespaces=())



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.api_token_uuid import ApiTokenUuid
from ..models.uuid import Uuid


class ApiToken(BaseModel):
    """An API token.

    These are used to authenticate users with Bearer authentication."""

    created_at: datetime.datetime

    id: Uuid

    is_valid: bool

    label: Optional[str] = None

    token: ApiTokenUuid

    updated_at: datetime.datetime

    user_id: Uuid

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class ModelingAppEventType(str, Enum):
    """Type for modeling-app events"""  # noqa: E501

    """# This event is sent before the modeling app or project is closed. The attachment should contain the contents of the most recent successful compile. """  # noqa: E501
    SUCCESSFUL_COMPILE_BEFORE_CLOSE = "successful_compile_before_close"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class EntityType(str, Enum):
    """The type of entity"""  # noqa: E501

    ENTITY = "entity"
    OBJECT = "object"
    PATH = "path"
    CURVE = "curve"
    SOLID2D = "solid2d"
    SOLID3D = "solid3d"
    EDGE = "edge"
    FACE = "face"
    PLANE = "plane"
    VERTEX = "vertex"

    def __str__(self) -> str:
        return str(self.value)



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.plan_interval import PlanInterval


class OptionFlat(BaseModel):
    """A flat price that we publicly list."""

    interval: PlanInterval

    price: float

    type: Literal["flat"] = "flat"

    model_config = ConfigDict(protected_namespaces=())


class OptionPerUser(BaseModel):
    """A per user price that we publicly list."""

    interval: PlanInterval

    price: float

    type: Literal["per_user"] = "per_user"

    model_config = ConfigDict(protected_namespaces=())


class OptionEnterprise(BaseModel):
    """Enterprise: The price is not listed and the user needs to contact sales."""

    type: Literal["enterprise"] = "enterprise"

    model_config = ConfigDict(protected_namespaces=())


SubscriptionTierPrice = RootModel[
    Annotated[
        Union[
            OptionFlat,
            OptionPerUser,
            OptionEnterprise,
        ],
        Field(discriminator="type"),
    ]
]



from typing import List

from pydantic import BaseModel, ConfigDict


class PathGetCurveUuidsForVertices(BaseModel):
    """The response from the `PathGetCurveUuidsForVertices` command."""

    curve_ids: List[str]

    model_config = ConfigDict(protected_namespaces=())



from typing import Dict, Optional

from pydantic import BaseModel, ConfigDict


class Coupon(BaseModel):
    """The resource representing a Coupon."""

    amount_off: Optional[float] = None

    deleted: bool = False

    id: Optional[str] = None

    metadata: Dict[str, str] = {}

    name: Optional[str] = None

    percent_off: Optional[float] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.error import Error
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/user/payment/tax".format(
        client.base_url,
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Error]:
    return None
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(*, response: httpx.Response) -> Response[Optional[Error]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
) -> Optional[Error]:
    """This endpoint requires authentication by any Zoo user. It will return an error if the user's information is not valid for automatic tax. Otherwise, it will return an empty successful response."""  # noqa: E501

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
) -> Optional[Error]:
    """This endpoint requires authentication by any Zoo user. It will return an error if the user's information is not valid for automatic tax. Otherwise, it will return an empty successful response."""  # noqa: E501

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from enum import Enum


class AccountProvider(str, Enum):
    """An account provider."""  # noqa: E501

    """# The Apple account provider. """  # noqa: E501
    APPLE = "apple"
    """# The Discord account provider. """  # noqa: E501
    DISCORD = "discord"
    """# The Google account provider. """  # noqa: E501
    GOOGLE = "google"
    """# The GitHub account provider. """  # noqa: E501
    GITHUB = "github"
    """# The Microsoft account provider. """  # noqa: E501
    MICROSOFT = "microsoft"
    """# The SAML account provider. """  # noqa: E501
    SAML = "saml"
    """# The Tencent QQ account provider. """  # noqa: E501
    TENCENT = "tencent"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitVolume(str, Enum):
    """The valid types of volume units."""  # noqa: E501

    """# Cubic centimeters (cc or cm³) <https://en.wikipedia.org/wiki/Cubic_centimeter> """  # noqa: E501
    CM3 = "cm3"
    """# Cubic feet (ft³) <https://en.wikipedia.org/wiki/Cubic_foot> """  # noqa: E501
    FT3 = "ft3"
    """# Cubic inches (cu in or in³) <https://en.wikipedia.org/wiki/Cubic_inch> """  # noqa: E501
    IN3 = "in3"
    """# Cubic meters (m³) <https://en.wikipedia.org/wiki/Cubic_meter> """  # noqa: E501
    M3 = "m3"
    """# Cubic yards (yd³) <https://en.wikipedia.org/wiki/Cubic_yard> """  # noqa: E501
    YD3 = "yd3"
    """# US Fluid Ounces (fl oz) <https://en.wikipedia.org/wiki/Fluid_ounce> """  # noqa: E501
    USFLOZ = "usfloz"
    """# US Gallons (gal US) <https://en.wikipedia.org/wiki/Gallon> """  # noqa: E501
    USGAL = "usgal"
    """# Liters (l) <https://en.wikipedia.org/wiki/Litre> """  # noqa: E501
    L = "l"
    """# Milliliters (ml) <https://en.wikipedia.org/wiki/Litre> """  # noqa: E501
    ML = "ml"

    def __str__(self) -> str:
        return str(self.value)



from typing import Optional

from pydantic import BaseModel, ConfigDict


class OutputFile(BaseModel):
    """Output file contents.

    <details><summary>JSON schema</summary>

    ```json { \"description\": \"Output file contents.\", \"type\": \"object\", \"properties\": { \"contents\": { \"description\": \"The contents of the file. This is base64 encoded so we can ensure it is UTF-8 for JSON.\", \"type\": \"string\" }, \"name\": { \"description\": \"The name of the file.\", \"default\": \"\", \"type\": \"string\" } } } ``` </details>"""

    contents: Optional[str] = None

    name: str = ""

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d


class FaceIsPlanar(BaseModel):
    """Surface-local planar axes (if available)"""

    origin: Optional[Point3d] = None

    x_axis: Optional[Point3d] = None

    y_axis: Optional[Point3d] = None

    z_axis: Optional[Point3d] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class OAuth2ClientInfo(BaseModel):
    """Information about an OAuth 2.0 client."""

    csrf_token: Optional[str] = None

    pkce_code_verifier: Optional[str] = None

    url: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Union

from pydantic import RootModel

from .failure_web_socket_response import FailureWebSocketResponse
from .success_web_socket_response import SuccessWebSocketResponse

WebSocketResponse = RootModel[
    Union[
        SuccessWebSocketResponse,
        FailureWebSocketResponse,
    ]
]



from enum import Enum


class Method(str, Enum):
    """The Request Method (VERB)

    This type also contains constants for a number of common HTTP methods such as GET, POST, etc.

    Currently includes 8 variants representing the 8 methods defined in [RFC 7230](https://tools.ietf.org/html/rfc7231#section-4.1), plus PATCH, and an Extension variant for all extensions."""  # noqa: E501

    """# The `OPTIONS` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.2.1). """  # noqa: E501
    OPTIONS = "OPTIONS"
    """# The `GET` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.3.1). """  # noqa: E501
    GET = "GET"
    """# The `POST` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.3.1). """  # noqa: E501
    POST = "POST"
    """# The `PUT` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.3.1). """  # noqa: E501
    PUT = "PUT"
    """# The `DELETE` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.3.5). """  # noqa: E501
    DELETE = "DELETE"
    """# The `HEAD` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.3.2). """  # noqa: E501
    HEAD = "HEAD"
    """# The `TRACE` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.3). """  # noqa: E501
    TRACE = "TRACE"
    """# The `CONNECT` method as defined in [RFC 7231](https://tools.ietf.org/html/rfc7231#section-4.3.6). """  # noqa: E501
    CONNECT = "CONNECT"
    """# The `PATCH` method as defined in [RFC 5789](https://tools.ietf.org/html/rfc5789). """  # noqa: E501
    PATCH = "PATCH"
    """# A catch all. """  # noqa: E501
    EXTENSION = "EXTENSION"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict


class DeviceAuthVerifyParams(BaseModel):
    """The request parameters to verify the `user_code` for the OAuth 2.0 Device Authorization Grant."""

    user_code: str

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.modeling_app_individual_subscription_tier import (
    ModelingAppIndividualSubscriptionTier,
)


class ZooProductSubscriptionsUserRequest(BaseModel):
    """A struct of Zoo product subscriptions a user can request."""

    modeling_app: ModelingAppIndividualSubscriptionTier = "free"  # type: ignore

    pay_annually: Optional[bool] = None

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.annotation_text_alignment_x import AnnotationTextAlignmentX
from ..models.annotation_text_alignment_y import AnnotationTextAlignmentY


class AnnotationTextOptions(BaseModel):
    """Options for annotation text"""

    point_size: int

    text: str

    x: AnnotationTextAlignmentX

    y: AnnotationTextAlignmentY

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class UpdateUser(BaseModel):
    """The user-modifiable parts of a User."""

    company: Optional[str] = None

    discord: Optional[str] = None

    first_name: Optional[str] = None

    github: Optional[str] = None

    image: str

    last_name: Optional[str] = None

    phone: str = ""

    model_config = ConfigDict(protected_namespaces=())



from typing import List

from pydantic import BaseModel, ConfigDict


class MouseClick(BaseModel):
    """The response from the `MouseClick` command."""

    entities_modified: List[str]

    entities_selected: List[str]

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.entity_type import EntityType


class GetEntityType(BaseModel):
    """The response from the `GetEntityType` command."""

    entity_type: EntityType

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class FileExportFormat(str, Enum):
    """The valid types of output file formats."""  # noqa: E501

    """# Autodesk Filmbox (FBX) format. <https://en.wikipedia.org/wiki/FBX> """  # noqa: E501
    FBX = "fbx"
    """# Binary glTF 2.0.

This is a single binary with .glb extension.

This is better if you want a compressed format as opposed to the human readable glTF that lacks compression. """  # noqa: E501
    GLB = "glb"
    """# glTF 2.0. Embedded glTF 2.0 (pretty printed).

Single JSON file with .gltf extension binary data encoded as base64 data URIs.

The JSON contents are pretty printed.

It is human readable, single file, and you can view the diff easily in a git commit. """  # noqa: E501
    GLTF = "gltf"
    """# The OBJ file format. <https://en.wikipedia.org/wiki/Wavefront_.obj_file> It may or may not have an an attached material (mtl // mtllib) within the file, but we interact with it as if it does not. """  # noqa: E501
    OBJ = "obj"
    """# The PLY file format. <https://en.wikipedia.org/wiki/PLY_(file_format)> """  # noqa: E501
    PLY = "ply"
    """# The STEP file format. <https://en.wikipedia.org/wiki/ISO_10303-21> """  # noqa: E501
    STEP = "step"
    """# The STL file format. <https://en.wikipedia.org/wiki/STL_(file_format)> """  # noqa: E501
    STL = "stl"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitMass(str, Enum):
    """The valid types of mass units."""  # noqa: E501

    """# Grams <https://en.wikipedia.org/wiki/Gram> """  # noqa: E501
    G = "g"
    """# Kilograms <https://en.wikipedia.org/wiki/Kilogram> """  # noqa: E501
    KG = "kg"
    """# Pounds <https://en.wikipedia.org/wiki/Pound_(mass)> """  # noqa: E501
    LB = "lb"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.unit_angle import UnitAngle


class Angle(BaseModel):
    """An angle, with a specific unit."""

    unit: UnitAngle

    value: float

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class MetaClusterInfo(BaseModel):
    """Jetstream statistics."""

    cluster_size: int = 0

    leader: str = ""

    name: str = ""

    model_config = ConfigDict(protected_namespaces=())



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated


class OptionIndividual(BaseModel):
    """A subscription tier that can be applied to individuals only."""

    type: Literal["individual"] = "individual"

    model_config = ConfigDict(protected_namespaces=())


class OptionOrganization(BaseModel):
    """An subscription tier that can be applied to organizations only."""

    saml_sso: bool

    type: Literal["organization"] = "organization"

    model_config = ConfigDict(protected_namespaces=())


SubscriptionTierType = RootModel[
    Annotated[
        Union[
            OptionIndividual,
            OptionOrganization,
        ],
        Field(discriminator="type"),
    ]
]



from enum import Enum


class AnnotationType(str, Enum):
    """The type of annotation"""  # noqa: E501

    """# 2D annotation type (screen or planar space) """  # noqa: E501
    T2D = "t2d"
    """# 3D annotation type """  # noqa: E501
    T3D = "t3d"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class SceneSelectionType(str, Enum):
    """The type of scene selection change"""  # noqa: E501

    """# Replaces the selection """  # noqa: E501
    REPLACE = "replace"
    """# Adds to the selection """  # noqa: E501
    ADD = "add"
    """# Removes from the selection """  # noqa: E501
    REMOVE = "remove"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.unit_density import UnitDensity


class Density(BaseModel):
    """The density response."""

    density: float

    output_unit: UnitDensity

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.cache_metadata import CacheMetadata
from ..models.connection import Connection
from ..models.environment import Environment
from ..models.file_system_metadata import FileSystemMetadata


class Metadata(BaseModel):
    """Metadata about our currently running server.

    This is mostly used for internal purposes and debugging."""

    cache: CacheMetadata

    environment: Environment

    fs: FileSystemMetadata

    git_hash: str

    pubsub: Connection

    model_config = ConfigDict(protected_namespaces=())



"""Contains methods for accessing the service_accounts API paths: Service accounts allow organizations to call the API. Organization admins can create, delete, and list the service accounts for their org. Service accounts are scoped to an organization not individual users, these are better to use for automations than individual API tokens, since they won't stop working when an individual leaves the company."""  # noqa: E501



from pydantic import BaseModel, ConfigDict


class Point4d(BaseModel):
    """A point in homogeneous (4D) space"""

    w: float

    x: float

    y: float

    z: float

    model_config = ConfigDict(protected_namespaces=())



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.uuid import Uuid


class VerificationTokenResponse(BaseModel):
    """A verification token response."""

    created_at: datetime.datetime

    expires: datetime.datetime

    id: Uuid

    identifier: Optional[str] = None

    saml_redirect_url: Optional[str] = None

    updated_at: datetime.datetime

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class FileImportFormat(str, Enum):
    """The valid types of source file formats."""  # noqa: E501

    """# Autodesk Filmbox (FBX) format. <https://en.wikipedia.org/wiki/FBX> """  # noqa: E501
    FBX = "fbx"
    """# glTF 2.0. """  # noqa: E501
    GLTF = "gltf"
    """# The OBJ file format. <https://en.wikipedia.org/wiki/Wavefront_.obj_file> It may or may not have an an attached material (mtl // mtllib) within the file, but we interact with it as if it does not. """  # noqa: E501
    OBJ = "obj"
    """# The PLY file format. <https://en.wikipedia.org/wiki/PLY_(file_format)> """  # noqa: E501
    PLY = "ply"
    """# SolidWorks part (SLDPRT) format. """  # noqa: E501
    SLDPRT = "sldprt"
    """# The STEP file format. <https://en.wikipedia.org/wiki/ISO_10303-21> """  # noqa: E501
    STEP = "step"
    """# The STL file format. <https://en.wikipedia.org/wiki/STL_(file_format)> """  # noqa: E501
    STL = "stl"

    def __str__(self) -> str:
        return str(self.value)



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.org_member import OrgMember


class OrgMemberResultsPage(BaseModel):
    """A single page of results"""

    items: List[OrgMember]

    next_page: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class Axis(str, Enum):
    """Co-ordinate axis specifier.

    See [cglearn.eu] for background reading.

    [cglearn.eu]: https://cglearn.eu/pub/computer-graphics/introduction-to-geometry#material-coordinate-systems-1"""  # noqa: E501

    """# 'Y' axis. """  # noqa: E501
    Y = "y"
    """# 'Z' axis. """  # noqa: E501
    Z = "z"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.api_call_status import ApiCallStatus
from ..models.unit_force import UnitForce
from ..models.uuid import Uuid


class UnitForceConversion(BaseModel):
    """Result of converting between units."""

    completed_at: Optional[datetime.datetime] = None

    created_at: datetime.datetime

    error: Optional[str] = None

    id: Uuid

    input: float = 0.0

    input_unit: UnitForce

    output: Optional[float] = None

    output_unit: UnitForce

    started_at: Optional[datetime.datetime] = None

    status: ApiCallStatus

    updated_at: datetime.datetime

    user_id: Uuid

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class PaymentMethodCardChecks(BaseModel):
    """Card checks."""

    address_line1_check: Optional[str] = None

    address_postal_code_check: Optional[str] = None

    cvc_check: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.error import Error
from ...models.uuid import Uuid
from ...types import Response


def _get_kwargs(
    provider_id: Uuid,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Dict[str, Any]:
    url = "{}/auth/saml/provider/{provider_id}/login".format(
        client.base_url,
        provider_id=provider_id,
    )  # noqa: E501

    if callback_url is not None:
        if "?" in url:
            url = url + "&callback_url=" + str(callback_url)
        else:
            url = url + "?callback_url=" + str(callback_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Error]:
    return None
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(*, response: httpx.Response) -> Response[Optional[Error]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    provider_id: Uuid,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        provider_id=provider_id,
        callback_url=callback_url,
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    provider_id: Uuid,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Optional[Error]:
    """The UI uses this to avoid having to ask the API anything about the IdP. It already knows the SAML IdP ID from the path, so it can just link to this path and rely on the API to redirect to the actual IdP."""  # noqa: E501

    return sync_detailed(
        provider_id=provider_id,
        callback_url=callback_url,
        client=client,
    ).parsed


async def asyncio_detailed(
    provider_id: Uuid,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        provider_id=provider_id,
        callback_url=callback_url,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    provider_id: Uuid,
    *,
    client: Client,
    callback_url: Optional[str] = None,
) -> Optional[Error]:
    """The UI uses this to avoid having to ask the API anything about the IdP. It already knows the SAML IdP ID from the path, so it can just link to this path and rely on the API to redirect to the actual IdP."""  # noqa: E501

    return (
        await asyncio_detailed(
            provider_id=provider_id,
            callback_url=callback_url,
            client=client,
        )
    ).parsed



from pydantic import BaseModel, ConfigDict

from ..models.unit_volume import UnitVolume


class Volume(BaseModel):
    """The volume response."""

    output_unit: UnitVolume

    volume: float

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class PathGetSketchTargetUuid(BaseModel):
    """The response from the `PathGetSketchTargetUuid` command."""

    target_id: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class MlFeedback(str, Enum):
    """Human feedback on an ML response."""  # noqa: E501

    """# Thumbs up. """  # noqa: E501
    THUMBS_UP = "thumbs_up"
    """# Thumbs down. """  # noqa: E501
    THUMBS_DOWN = "thumbs_down"
    """# Accepted. """  # noqa: E501
    ACCEPTED = "accepted"
    """# Rejected. """  # noqa: E501
    REJECTED = "rejected"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.jetstream_api_stats import JetstreamApiStats


class JetstreamStats(BaseModel):
    """Jetstream statistics."""

    accounts: int = 0

    api: JetstreamApiStats = {"errors": 0, "inflight": 0, "total": 0}  # type: ignore

    ha_assets: int = 0

    memory: int = 0

    reserved_memory: int = 0

    reserved_store: int = 0

    store: int = 0

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.device_access_token_uuid import DeviceAccessTokenUuid


class TokenRevokeRequestForm(BaseModel):
    """The request parameters for the OAuth 2.0 token revocation flow."""

    client_id: str

    client_secret: Optional[str] = None

    token: DeviceAccessTokenUuid

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class AnnotationTextAlignmentY(str, Enum):
    """Vertical Text alignment"""  # noqa: E501

    BOTTOM = "bottom"
    CENTER = "center"
    TOP = "top"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.annotation_line_end import AnnotationLineEnd


class AnnotationLineEndOptions(BaseModel):
    """Options for annotation text"""

    end: AnnotationLineEnd

    start: AnnotationLineEnd

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class ModelingSessionData(BaseModel):
    """Successful Websocket response."""

    api_call_id: str

    model_config = ConfigDict(protected_namespaces=())



from typing import Dict, List, Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.client_metrics import ClientMetrics
from ..models.modeling_cmd import ModelingCmd
from ..models.modeling_cmd_id import ModelingCmdId
from ..models.modeling_cmd_req import ModelingCmdReq
from ..models.rtc_ice_candidate_init import RtcIceCandidateInit
from ..models.rtc_session_description import RtcSessionDescription


class OptionTrickleIce(BaseModel):
    """The trickle ICE candidate request."""

    candidate: RtcIceCandidateInit

    type: Literal["trickle_ice"] = "trickle_ice"

    model_config = ConfigDict(protected_namespaces=())


class OptionSdpOffer(BaseModel):
    """The SDP offer request."""

    offer: RtcSessionDescription

    type: Literal["sdp_offer"] = "sdp_offer"

    model_config = ConfigDict(protected_namespaces=())


class OptionModelingCmdReq(BaseModel):
    """The modeling command request."""

    cmd: ModelingCmd

    cmd_id: ModelingCmdId

    type: Literal["modeling_cmd_req"] = "modeling_cmd_req"

    model_config = ConfigDict(protected_namespaces=())


class OptionModelingCmdBatchReq(BaseModel):
    """A sequence of modeling requests. If any request fails, following requests will not be tried."""

    batch_id: ModelingCmdId

    requests: List[ModelingCmdReq]

    responses: bool = False

    type: Literal["modeling_cmd_batch_req"] = "modeling_cmd_batch_req"

    model_config = ConfigDict(protected_namespaces=())


class OptionPing(BaseModel):
    """The client-to-server Ping to ensure the WebSocket stays alive."""

    type: Literal["ping"] = "ping"

    model_config = ConfigDict(protected_namespaces=())


class OptionMetricsResponse(BaseModel):
    """The response to a metrics collection request from the server."""

    metrics: ClientMetrics

    type: Literal["metrics_response"] = "metrics_response"

    model_config = ConfigDict(protected_namespaces=())


class OptionHeaders(BaseModel):
    """Authentication header request."""

    headers: Dict[str, str]

    type: Literal["headers"] = "headers"

    model_config = ConfigDict(protected_namespaces=())


WebSocketRequest = RootModel[
    Annotated[
        Union[
            OptionTrickleIce,
            OptionSdpOffer,
            OptionModelingCmdReq,
            OptionModelingCmdBatchReq,
            OptionPing,
            OptionMetricsResponse,
            OptionHeaders,
        ],
        Field(discriminator="type"),
    ]
]



from pydantic import BaseModel, ConfigDict


class EntityGetNumChildren(BaseModel):
    """The response from the `EntityGetNumChildren` command."""

    num: int

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, Type, TypeVar

import attr

VI = TypeVar("VI", bound="Empty")


@attr.s(auto_attribs=True)
class Empty:
    def __str__(self) -> str:
        return ""

    @classmethod
    def from_dict(cls: Type[VI], src_dict: Dict[str, Any]) -> Any:
        return {}



from pydantic import BaseModel, ConfigDict


class StoreCouponParams(BaseModel):
    """The parameters for a new store coupon."""

    percent_off: int

    model_config = ConfigDict(protected_namespaces=())



import json
import os
import time
import uuid
from typing import Optional, Union

import pytest
from websockets.exceptions import ConnectionClosedError

from .api.api_tokens import list_api_tokens_for_user
from .api.file import (
    create_file_center_of_mass,
    create_file_conversion,
    create_file_mass,
    create_file_volume,
)
from .api.meta import ping
from .api.ml import create_text_to_cad, get_text_to_cad_model_for_user
from .api.modeling import modeling_commands_ws
from .api.users import get_user_self, list_users_extended
from .client import ClientFromEnv
from .models import (
    ApiCallStatus,
    ApiTokenResultsPage,
    Axis,
    AxisDirectionPair,
    CreatedAtSortMode,
    Direction,
    Error,
    ExtendedUserResultsPage,
    FileCenterOfMass,
    FileConversion,
    FileExportFormat,
    FileImportFormat,
    FileMass,
    FileVolume,
    ImageFormat,
    ImportFile,
    InputFormat,
    ModelingCmd,
    ModelingCmdId,
    Pong,
    PostEffectType,
    System,
    TextToCad,
    TextToCadCreateBody,
    UnitDensity,
    UnitLength,
    UnitMass,
    UnitVolume,
    User,
    WebSocketRequest,
    WebSocketResponse,
)
from .models.input_format import OptionObj
from .models.modeling_cmd import (
    OptionDefaultCameraFocusOn,
    OptionImportFiles,
    OptionStartPath,
    OptionTakeSnapshot,
)
from .models.web_socket_request import OptionModelingCmdReq
from .types import Unset


def test_get_session():
    # Create our client.
    client = ClientFromEnv()

    # Get the session.
    session: Union[User, Error, None] = get_user_self.sync(client=client)

    assert isinstance(session, User)

    print(f"Session: {session}")


@pytest.mark.asyncio
async def test_get_api_tokens_async():
    # Create our client.
    client = ClientFromEnv()

    # List API tokens.
    fc: Union[ApiTokenResultsPage, Error, None] = list_api_tokens_for_user.sync(
        client=client, sort_by=CreatedAtSortMode.CREATED_AT_ASCENDING
    )

    assert isinstance(fc, ApiTokenResultsPage)

    print(f"fc: {fc}")


@pytest.mark.asyncio
async def test_get_session_async():
    # Create our client.
    client = ClientFromEnv()

    # Get the session.
    session: Union[User, Error, None] = await get_user_self.asyncio(client=client)

    assert isinstance(session, User)

    print(f"Session: {session}")


def test_ping():
    # Create our client.
    client = ClientFromEnv()

    # Get the message.
    message: Union[Pong, Error, None] = ping.sync(client=client)

    assert isinstance(message, Pong)

    print(f"Message: {message}")


@pytest.mark.asyncio
async def test_ping_async():
    # Create our client.
    client = ClientFromEnv()

    # Get the message.
    message: Union[Pong, Error, None] = await ping.asyncio(client=client)

    assert isinstance(message, Pong)

    print(f"Message: {message}")


def test_file_convert_stl():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.stl"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Optional[Union[FileConversion, Error]] = create_file_conversion.sync(
        client=client,
        body=content,
        src_format=FileImportFormat.STL,
        output_format=FileExportFormat.OBJ,
    )

    assert isinstance(result, FileConversion)

    fc: FileConversion = result

    print(f"FileConversion: {fc}")

    assert fc.id is not None
    assert fc.status == ApiCallStatus.COMPLETED

    print(f"FileConversion: {fc}")

    assert not isinstance(fc.outputs, Unset)
    assert fc.outputs is not None

    # Make sure the bytes are not empty.
    for key, value in fc.outputs.items():
        assert len(value) > 0


@pytest.mark.asyncio
async def test_file_convert_stl_async():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.stl"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Optional[
        Union[FileConversion, Error]
    ] = await create_file_conversion.asyncio(
        client=client,
        body=content,
        src_format=FileImportFormat.STL,
        output_format=FileExportFormat.OBJ,
    )

    assert isinstance(result, FileConversion)

    fc: FileConversion = result

    print(f"FileConversion: {fc}")

    assert fc.id is not None
    assert fc.status == ApiCallStatus.COMPLETED

    print(f"FileConversion: {fc}")

    assert not isinstance(fc.outputs, Unset)
    assert fc.outputs is not None

    # Make sure the bytes are not empty.
    for key, value in fc.outputs.items():
        assert len(value) > 0


@pytest.mark.asyncio
async def test_file_convert_obj_async():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/ORIGINALVOXEL-3.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Optional[
        Union[FileConversion, Error]
    ] = await create_file_conversion.asyncio(
        client=client,
        body=content,
        src_format=FileImportFormat.OBJ,
        output_format=FileExportFormat.STL,
    )

    assert isinstance(result, FileConversion)

    fc: FileConversion = result

    print(f"FileConversion: {fc}")

    assert fc.id is not None
    assert fc.status == ApiCallStatus.COMPLETED

    print(f"FileConversion: {fc}")

    assert not isinstance(fc.outputs, Unset)
    assert fc.outputs is not None

    # Make sure the bytes are not empty.
    for key, value in fc.outputs.items():
        assert len(value) > 0


def test_file_mass():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Union[FileMass, Error, None] = create_file_mass.sync(
        client=client,
        body=content,
        src_format=FileImportFormat.OBJ,
        material_density=1.0,
        material_density_unit=UnitDensity.KG_M3,
        output_unit=UnitMass.G,
    )

    assert isinstance(result, FileMass)

    fm: FileMass = result

    print(f"FileMass: {fm}")

    assert fm.id is not None
    assert fm.mass is not None

    assert fm.model_dump_json() is not None

    assert fm.status == ApiCallStatus.COMPLETED


def test_file_volume():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Union[FileVolume, Error, None] = create_file_volume.sync(
        client=client,
        body=content,
        src_format=FileImportFormat.OBJ,
        output_unit=UnitVolume.CM3,
    )

    assert isinstance(result, FileVolume)

    fv: FileVolume = result

    print(f"FileVolume: {fv}")

    assert fv.id is not None
    assert fv.volume is not None

    assert fv.model_dump_json() is not None

    assert fv.status == ApiCallStatus.COMPLETED


def test_file_center_of_mass():
    # Create our client.
    client = ClientFromEnv()

    dir_path = os.path.dirname(os.path.realpath(__file__))
    file = open(os.path.join(dir_path, "../assets/testing.obj"), "rb")
    content = file.read()
    file.close()

    # Get the fc.
    result: Union[FileCenterOfMass, Error, None] = create_file_center_of_mass.sync(
        client=client,
        body=content,
        src_format=FileImportFormat.OBJ,
        output_unit=UnitLength.CM,
    )

    assert isinstance(result, FileCenterOfMass)

    fv: FileCenterOfMass = result

    print(f"FileCenterOfMass: {fv}")

    assert fv.id is not None
    assert fv.center_of_mass is not None

    assert fv.model_dump_json() is not None

    assert fv.status == ApiCallStatus.COMPLETED


def test_list_users():
    # Create our client.
    client = ClientFromEnv()

    response: Union[ExtendedUserResultsPage, Error, None] = list_users_extended.sync(
        sort_by=CreatedAtSortMode.CREATED_AT_DESCENDING, client=client, limit=10
    )

    assert isinstance(response, ExtendedUserResultsPage)

    print(f"ExtendedUserResultsPage: {response}")


def test_ws_simple():
    # Create our client.
    client = ClientFromEnv()

    # Connect to the websocket.
    with modeling_commands_ws.WebSocket(
        client=client,
        fps=30,
        show_grid=False,
        post_effect=PostEffectType.NOEFFECT,
        unlocked_framerate=False,
        video_res_height=360,
        video_res_width=480,
        webrtc=False,
    ) as websocket:
        # Send a message.
        id = uuid.uuid4()
        req = WebSocketRequest(
            OptionModelingCmdReq(
                cmd=ModelingCmd(OptionStartPath()), cmd_id=ModelingCmdId(id)
            )
        )
        websocket.send(req)

        # Get the messages.
        while True:
            message = websocket.recv()
            print(json.dumps(message.model_dump_json()))
            break


def test_ws_import():
    max_retries = 3
    for attempt in range(1, max_retries + 1):
        try:
            # Create our client.
            client = ClientFromEnv()

            # Connect to the websocket.
            with modeling_commands_ws.WebSocket(
                client=client,
                fps=30,
                post_effect=PostEffectType.NOEFFECT,
                show_grid=False,
                unlocked_framerate=False,
                video_res_height=360,
                video_res_width=480,
                webrtc=False,
            ) as websocket:
                # read the content of the file
                dir_path = os.path.dirname(os.path.realpath(__file__))
                file_name = "ORIGINALVOXEL-3.obj"
                file_path = os.path.join(dir_path, "..", "assets", file_name)
                with open(file_path, "rb") as file:
                    content = file.read()
                cmd_id = uuid.uuid4()
                ImportFile(data=content, path=file_name)
                # form the request
                req = WebSocketRequest(
                    OptionModelingCmdReq(
                        cmd=ModelingCmd(
                            OptionImportFiles(
                                files=[ImportFile(data=content, path=file_name)],
                                format=InputFormat(
                                    OptionObj(
                                        units=UnitLength.M,
                                        coords=System(
                                            forward=AxisDirectionPair(
                                                axis=Axis.Y,
                                                direction=Direction.NEGATIVE,
                                            ),
                                            up=AxisDirectionPair(
                                                axis=Axis.Z,
                                                direction=Direction.POSITIVE,
                                            ),
                                        ),
                                    )
                                ),
                            )
                        ),
                        cmd_id=ModelingCmdId(cmd_id),
                    )
                )
                # Import files request must be sent as binary, because the file contents might be binary.
                websocket.send_binary(req)

                # Get the success message.
                for message in websocket:
                    message_dict = message.model_dump()
                    if message_dict["success"] is not True:
                        raise Exception(message_dict)
                    elif message_dict["resp"]["type"] != "modeling":
                        continue
                    elif (
                        message_dict["resp"]["data"]["modeling_response"]["type"]
                        != "import_files"
                    ):
                        # We have a modeling command response.
                        # Make sure its the import files response.
                        raise Exception(message_dict)
                    else:
                        # Okay we have the import files response.
                        # Break since now we know it was a success.
                        object_id = str(
                            message_dict["resp"]["data"]["modeling_response"]["data"][
                                "object_id"
                            ]
                        )
                        break

                # Now we want to focus on the object.
                cmd_id = uuid.uuid4()
                # form the request
                req = WebSocketRequest(
                    OptionModelingCmdReq(
                        cmd=ModelingCmd(OptionDefaultCameraFocusOn(uuid=object_id)),
                        cmd_id=ModelingCmdId(cmd_id),
                    )
                )
                websocket.send(req)

                # Get the success message.
                for message in websocket:
                    message_dict = message.model_dump()
                    if message_dict["success"] is not True:
                        raise Exception(message_dict)
                    elif message_dict["resp"]["type"] != "modeling":
                        continue
                    elif message_dict["request_id"] == str(cmd_id):
                        # We got a success response for our cmd.
                        break
                    else:
                        raise Exception(message_dict)

                # Now we want to snapshot as a png.
                cmd_id = uuid.uuid4()
                # form the request
                req = WebSocketRequest(
                    OptionModelingCmdReq(
                        cmd=ModelingCmd(OptionTakeSnapshot(format=ImageFormat.PNG)),
                        cmd_id=ModelingCmdId(cmd_id),
                    )
                )
                websocket.send(req)

                # Get the success message.
                for message in websocket:
                    message_dict = message.model_dump()
                    if message_dict["success"] is not True:
                        raise Exception(message_dict)
                    elif message_dict["resp"]["type"] != "modeling":
                        continue
                    elif (
                        message_dict["resp"]["data"]["modeling_response"]["type"]
                        != "take_snapshot"
                    ):
                        # Make sure its the correct response.
                        raise Exception(message_dict)
                    else:
                        # Okay we have the snapshot response.
                        # Break since now we know it was a success.
                        png_contents = message_dict["resp"]["data"][
                            "modeling_response"
                        ]["data"]["contents"]
                        break

                # Save the contents to a file.
                png_path = os.path.join(dir_path, "..", "assets", "snapshot.png")
                with open(png_path, "wb") as f:
                    f.write(png_contents)

                # Ensure the file is not empty.
                assert len(png_contents) > 0

                # Ensure the file exists.
                assert os.path.exists(png_path)

            # Exit the retry loop on success
            break

        except ConnectionClosedError:
            if attempt < max_retries:
                print(
                    f"ConnectionClosedError encountered on attempt {attempt}/{max_retries}. Retrying..."
                )
            else:
                # After max retries, re-raise the exception to fail the test
                print(
                    f"ConnectionClosedError encountered on attempt {attempt}/{max_retries}. No more retries left."
                )
                raise


def test_serialize_deserialize():
    json_str = """{"success":true,"request_id":"16a06065-6ca3-4a96-a042-d0bec6b161a6","resp":{"type":"modeling","data":{"modeling_response":{"type":"import_files","data":{"object_id":"f61ac02e-77bd-468f-858f-fd4141a26acd"}}}}}"""
    d = json.loads(json_str)
    print(d)
    message = WebSocketResponse(**d)
    model_dump = message.model_dump()
    print(model_dump)
    assert model_dump["success"] is True  # type: ignore
    assert model_dump["request_id"] == "16a06065-6ca3-4a96-a042-d0bec6b161a6"  # type: ignore
    assert model_dump["resp"]["type"] == "modeling"  # type: ignore
    assert model_dump["resp"]["data"]["modeling_response"]["type"] == "import_files"  # type: ignore
    assert (
        model_dump["resp"]["data"]["modeling_response"]["data"]["object_id"]
        == "f61ac02e-77bd-468f-858f-fd4141a26acd"
    )  # type: ignore


def test_deserialize_null_request_id():
    json_str = """{"success":true,"request_id":null,"resp":{"type":"modeling_session_data","data":{"session":{"api_call_id":"91f7fd17-8846-4593-97ff-6400a81b8cdd"}}}}"""
    d = json.loads(json_str)
    print(d)
    message = WebSocketResponse(**d)
    model_dump = message.model_dump()
    print(model_dump)
    assert model_dump["success"] is True  # type: ignore
    assert model_dump["success"] is True  # type: ignore
    assert model_dump["request_id"] is None  # type: ignore
    assert model_dump["resp"]["type"] == "modeling_session_data"  # type: ignore
    assert (
        model_dump["resp"]["data"]["session"]["api_call_id"]
        == "91f7fd17-8846-4593-97ff-6400a81b8cdd"
    )  # type: ignore


def test_text_to_cad():
    # Create our client.
    client = ClientFromEnv()

    result: Optional[Union[TextToCad, Error]] = create_text_to_cad.sync(
        client=client,
        output_format=FileExportFormat.STEP,
        body=TextToCadCreateBody(
            prompt="a 2x4 lego",
        ),
        kcl=None,
    )

    if isinstance(result, Error) or result is None:
        print(result)
        raise Exception("Error in response")

    body: TextToCad = result

    # Poll the api until the status is completed.
    # Timeout after some seconds.
    start_time = time.time()
    while (
        body.status == ApiCallStatus.IN_PROGRESS or body.status == ApiCallStatus.QUEUED
    ) and time.time() - start_time < 120:
        result_status: Optional[Union[TextToCad, Error]] = (
            get_text_to_cad_model_for_user.sync(
                client=client,
                id=body.id,
            )
        )

        if isinstance(result_status, Error) or result_status is None:
            print(result_status)
            raise Exception("Error in response")

        body = result_status

    assert body.status == ApiCallStatus.COMPLETED



from enum import Enum


class UnitArea(str, Enum):
    """The valid types of area units."""  # noqa: E501

    """# Square centimeters <https://en.wikipedia.org/wiki/Square_centimeter> """  # noqa: E501
    CM2 = "cm2"
    """# Square decimeters <https://en.wikipedia.org/wiki/Square_decimeter> """  # noqa: E501
    DM2 = "dm2"
    """# Square feet <https://en.wikipedia.org/wiki/Square_foot> """  # noqa: E501
    FT2 = "ft2"
    """# Square inches <https://en.wikipedia.org/wiki/Square_inch> """  # noqa: E501
    IN2 = "in2"
    """# Square kilometers <https://en.wikipedia.org/wiki/Square_kilometer> """  # noqa: E501
    KM2 = "km2"
    """# Square meters <https://en.wikipedia.org/wiki/Square_meter> """  # noqa: E501
    M2 = "m2"
    """# Square millimeters <https://en.wikipedia.org/wiki/Square_millimeter> """  # noqa: E501
    MM2 = "mm2"
    """# Square yards <https://en.wikipedia.org/wiki/Square_mile> """  # noqa: E501
    YD2 = "yd2"

    def __str__(self) -> str:
        return str(self.value)



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.source_range_prompt import SourceRangePrompt


class TextToCadIterationBody(BaseModel):
    """Body for generating models from text."""

    original_source_code: str

    prompt: Optional[str] = None

    source_ranges: List[SourceRangePrompt]

    model_config = ConfigDict(protected_namespaces=())



from typing import List, Optional

from pydantic import BaseModel, ConfigDict


class IceServer(BaseModel):
    """Representation of an ICE server used for STUN/TURN Used to initiate WebRTC connections based on <https://developer.mozilla.org/en-US/docs/Web/API/RTCIceServer>"""

    credential: Optional[str] = None

    urls: List[str]

    username: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict

from ..models.ok_web_socket_response_data import OkWebSocketResponseData


class SuccessWebSocketResponse(BaseModel):
    """Successful Websocket response."""

    request_id: Optional[str] = None

    resp: OkWebSocketResponseData

    success: bool

    model_config = ConfigDict(protected_namespaces=())



from typing import Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated


class OptionDefaultScene(BaseModel):
    """Visit the default scene."""

    type: Literal["default_scene"] = "default_scene"

    model_config = ConfigDict(protected_namespaces=())


class OptionSceneByIndex(BaseModel):
    """Visit the indexed scene."""

    index: int

    type: Literal["scene_by_index"] = "scene_by_index"

    model_config = ConfigDict(protected_namespaces=())


class OptionSceneByName(BaseModel):
    """Visit the first scene with the given name."""

    name: str

    type: Literal["scene_by_name"] = "scene_by_name"

    model_config = ConfigDict(protected_namespaces=())


class OptionMeshByIndex(BaseModel):
    """Visit the indexed mesh."""

    index: int

    type: Literal["mesh_by_index"] = "mesh_by_index"

    model_config = ConfigDict(protected_namespaces=())


class OptionMeshByName(BaseModel):
    """Visit the first mesh with the given name."""

    name: str

    type: Literal["mesh_by_name"] = "mesh_by_name"

    model_config = ConfigDict(protected_namespaces=())


Selection = RootModel[
    Annotated[
        Union[
            OptionDefaultScene,
            OptionSceneByIndex,
            OptionSceneByName,
            OptionMeshByIndex,
            OptionMeshByName,
        ],
        Field(discriminator="type"),
    ]
]



from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d


class EngineUtilEvaluatePath(BaseModel):
    """The response of the `EngineUtilEvaluatePath` endpoint"""

    pos: Point3d

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class ClosePath(BaseModel):
    """The response from the `ClosePath` command."""

    face_id: str

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class ImportFile(BaseModel):
    """File to import into the current model. If you are sending binary data for a file, be sure to send the WebSocketRequest as binary/bson, not text/json."""

    data: bytes

    path: str

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class BlockReason(str, Enum):
    """The reason for blocking a user."""  # noqa: E501

    """# The user is missing a payment method and has exceeded their free API call credits for the month. """  # noqa: E501
    MISSING_PAYMENT_METHOD = "missing_payment_method"
    """# The users payment method has failed. """  # noqa: E501
    PAYMENT_METHOD_FAILED = "payment_method_failed"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Literal, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.batch_response import BatchResponse
from ..models.ice_server import IceServer
from ..models.modeling_session_data import ModelingSessionData
from ..models.ok_modeling_cmd_response import OkModelingCmdResponse
from ..models.raw_file import RawFile
from ..models.rtc_ice_candidate_init import RtcIceCandidateInit
from ..models.rtc_session_description import RtcSessionDescription


class IceServerInfoData(BaseModel):
    """"""

    ice_servers: List[IceServer]

    model_config = ConfigDict(protected_namespaces=())


class OptionIceServerInfo(BaseModel):
    """Information about the ICE servers."""

    data: IceServerInfoData

    type: Literal["ice_server_info"] = "ice_server_info"

    model_config = ConfigDict(protected_namespaces=())


class TrickleIceData(BaseModel):
    """"""

    candidate: RtcIceCandidateInit

    model_config = ConfigDict(protected_namespaces=())


class OptionTrickleIce(BaseModel):
    """The trickle ICE candidate response."""

    data: TrickleIceData

    type: Literal["trickle_ice"] = "trickle_ice"

    model_config = ConfigDict(protected_namespaces=())


class SdpAnswerData(BaseModel):
    """"""

    answer: RtcSessionDescription

    model_config = ConfigDict(protected_namespaces=())


class OptionSdpAnswer(BaseModel):
    """The SDP answer response."""

    data: SdpAnswerData

    type: Literal["sdp_answer"] = "sdp_answer"

    model_config = ConfigDict(protected_namespaces=())


class ModelingData(BaseModel):
    """"""

    modeling_response: OkModelingCmdResponse

    model_config = ConfigDict(protected_namespaces=())


class OptionModeling(BaseModel):
    """The modeling command response."""

    data: ModelingData

    type: Literal["modeling"] = "modeling"

    model_config = ConfigDict(protected_namespaces=())


class ModelingBatchData(BaseModel):
    """"""

    responses: Dict[str, BatchResponse]

    model_config = ConfigDict(protected_namespaces=())


class OptionModelingBatch(BaseModel):
    """Response to a ModelingBatch."""

    data: ModelingBatchData

    type: Literal["modeling_batch"] = "modeling_batch"

    model_config = ConfigDict(protected_namespaces=())


class ExportData(BaseModel):
    """"""

    files: List[RawFile]

    model_config = ConfigDict(protected_namespaces=())


class OptionExport(BaseModel):
    """The exported files."""

    data: ExportData

    type: Literal["export"] = "export"

    model_config = ConfigDict(protected_namespaces=())


class MetricsRequestData(BaseModel):
    """"""

    model_config = ConfigDict(protected_namespaces=())


class OptionMetricsRequest(BaseModel):
    """Request a collection of metrics, to include WebRTC."""

    data: MetricsRequestData

    type: Literal["metrics_request"] = "metrics_request"

    model_config = ConfigDict(protected_namespaces=())


class ModelingSessionDataData(BaseModel):
    """"""

    session: ModelingSessionData

    model_config = ConfigDict(protected_namespaces=())


class OptionModelingSessionData(BaseModel):
    """Data about the Modeling Session (application-level)."""

    data: ModelingSessionDataData

    type: Literal["modeling_session_data"] = "modeling_session_data"

    model_config = ConfigDict(protected_namespaces=())


class PongData(BaseModel):
    """"""

    model_config = ConfigDict(protected_namespaces=())


class OptionPong(BaseModel):
    """Pong response to a Ping message."""

    data: PongData

    type: Literal["pong"] = "pong"

    model_config = ConfigDict(protected_namespaces=())


OkWebSocketResponseData = RootModel[
    Annotated[
        Union[
            OptionIceServerInfo,
            OptionTrickleIce,
            OptionSdpAnswer,
            OptionModeling,
            OptionModelingBatch,
            OptionExport,
            OptionMetricsRequest,
            OptionModelingSessionData,
            OptionPong,
        ],
        Field(discriminator="type"),
    ]
]



from enum import Enum


class ModelingAppOrganizationSubscriptionTier(str, Enum):
    """The subscription tiers we offer for the Modeling App to organizations."""  # noqa: E501

    """# The team tier. """  # noqa: E501
    TEAM = "team"
    """# The enterprise tier. """  # noqa: E501
    ENTERPRISE = "enterprise"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict


class EntityGetChildUuid(BaseModel):
    """The response from the `EntityGetChildUuid` command."""

    entity_id: str

    model_config = ConfigDict(protected_namespaces=())



from typing import Optional

from pydantic import BaseModel, ConfigDict


class KclCodeCompletionParams(BaseModel):
    """Extra params for the completions."""

    language: str = ""

    next_indent: Optional[int] = None

    prompt_tokens: Optional[int] = None

    suffix_tokens: Optional[int] = None

    trim_by_indentation: bool = False

    model_config = ConfigDict(protected_namespaces=())



from typing import Literal, Optional, Union

from pydantic import BaseModel, ConfigDict, Field, RootModel
from typing_extensions import Annotated

from ..models.angle import Angle
from ..models.length_unit import LengthUnit
from ..models.point2d import Point2d
from ..models.point3d import Point3d


class OptionLine(BaseModel):
    """A straight line segment. Goes from the current path \"pen\" to the given endpoint."""

    end: Point3d

    relative: bool

    type: Literal["line"] = "line"

    model_config = ConfigDict(protected_namespaces=())


class OptionArc(BaseModel):
    """A circular arc segment. Arcs can be drawn clockwise when start > end."""

    center: Point2d

    end: Angle

    radius: LengthUnit

    relative: bool

    start: Angle

    type: Literal["arc"] = "arc"

    model_config = ConfigDict(protected_namespaces=())


class OptionBezier(BaseModel):
    """A cubic bezier curve segment. Start at the end of the current line, go through control point 1 and 2, then end at a given point."""

    control1: Point3d

    control2: Point3d

    end: Point3d

    relative: bool

    type: Literal["bezier"] = "bezier"

    model_config = ConfigDict(protected_namespaces=())


class OptionTangentialArc(BaseModel):
    """Adds a tangent arc from current pen position with the given radius and angle."""

    offset: Angle

    radius: LengthUnit

    type: Literal["tangential_arc"] = "tangential_arc"

    model_config = ConfigDict(protected_namespaces=())


class OptionTangentialArcTo(BaseModel):
    """Adds a tangent arc from current pen position to the new position. Arcs will choose a clockwise or counter-clockwise direction based on the arc end position."""

    angle_snap_increment: Optional[Angle] = None

    to: Point3d

    type: Literal["tangential_arc_to"] = "tangential_arc_to"

    model_config = ConfigDict(protected_namespaces=())


PathSegment = RootModel[
    Annotated[
        Union[
            OptionLine,
            OptionArc,
            OptionBezier,
            OptionTangentialArc,
            OptionTangentialArcTo,
        ],
        Field(discriminator="type"),
    ]
]



from pydantic import BaseModel, ConfigDict


class ApiCallQueryGroup(BaseModel):
    """A response for a query on the API call table that is grouped by something."""

    count: int

    query: str

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.user_org_role import UserOrgRole


class UpdateMemberToOrgBody(BaseModel):
    """Data for updating a member of an org."""

    role: UserOrgRole

    model_config = ConfigDict(protected_namespaces=())



from enum import Enum


class UnitCurrent(str, Enum):
    """The valid types of current units."""  # noqa: E501

    """# Amperes <https://en.wikipedia.org/wiki/Ampere> """  # noqa: E501
    AMPERES = "amperes"
    """# Microamperes <https://en.wikipedia.org/wiki/Microampere> """  # noqa: E501
    MICROAMPERES = "microamperes"
    """# Milliamperes <https://en.wikipedia.org/wiki/Milliampere> """  # noqa: E501
    MILLIAMPERES = "milliamperes"
    """# Nanoamperes <https://en.wikipedia.org/wiki/Nanoampere> """  # noqa: E501
    NANOAMPERES = "nanoamperes"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitPower(str, Enum):
    """The valid types of power units."""  # noqa: E501

    """# British thermal units (BTU) per minute <https://en.wikipedia.org/wiki/British_thermal_unit> """  # noqa: E501
    BTU_PER_MINUTE = "btu_per_minute"
    """# Horsepower (hp) <https://en.wikipedia.org/wiki/Horsepower> """  # noqa: E501
    HORSEPOWER = "horsepower"
    """# Kilowatts <https://en.wikipedia.org/wiki/Kilowatt> """  # noqa: E501
    KILOWATTS = "kilowatts"
    """# Metric horsepower (PS) <https://en.wikipedia.org/wiki/Horsepower#Metric_horsepower> """  # noqa: E501
    METRIC_HORSEPOWER = "metric_horsepower"
    """# Microwatts <https://en.wikipedia.org/wiki/Microwatt> """  # noqa: E501
    MICROWATTS = "microwatts"
    """# Millwatts <https://en.wikipedia.org/wiki/Milliwatt> """  # noqa: E501
    MILLIWATTS = "milliwatts"
    """# Watts <https://en.wikipedia.org/wiki/Watt> """  # noqa: E501
    WATTS = "watts"

    def __str__(self) -> str:
        return str(self.value)



from pydantic import BaseModel, ConfigDict

from ..models.point3d import Point3d


class FaceGetGradient(BaseModel):
    """The gradient (dFdu, dFdv) + normal vector on a brep face"""

    df_du: Point3d

    df_dv: Point3d

    normal: Point3d

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict

from ..models.length_unit import LengthUnit


class Point2d(BaseModel):
    """A point in 2D space"""

    x: LengthUnit

    y: LengthUnit

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class DeviceAuthRequestForm(BaseModel):
    """The request parameters for the OAuth 2.0 Device Authorization Grant flow."""

    client_id: str

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class CacheMetadata(BaseModel):
    """Metadata about our cache.

    This is mostly used for internal purposes and debugging."""

    ok: bool

    model_config = ConfigDict(protected_namespaces=())



from typing import List, Optional

from pydantic import BaseModel, ConfigDict

from ..models.source_range_prompt import SourceRangePrompt


class MlPromptMetadata(BaseModel):
    """Metadata for a ML prompt."""

    code: Optional[str] = None

    original_source_code: Optional[str] = None

    source_ranges: Optional[List[SourceRangePrompt]] = None

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.add_org_member import AddOrgMember
from ...models.error import Error
from ...models.org_member import OrgMember
from ...types import Response


def _get_kwargs(
    body: AddOrgMember,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/org/members".format(
        client.base_url,
    )  # noqa: E501

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "content": body.model_dump_json(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[OrgMember, Error]]:
    if response.status_code == 201:
        response_201 = OrgMember(**response.json())
        return response_201
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[OrgMember, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    body: AddOrgMember,
    *,
    client: Client,
) -> Response[Optional[Union[OrgMember, Error]]]:
    kwargs = _get_kwargs(
        body=body,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    body: AddOrgMember,
    *,
    client: Client,
) -> Optional[Union[OrgMember, Error]]:
    """If the user exists, this will add them to your org. If they do not exist, this will create a new user and add them to your org.

    In both cases the user gets an email that they have been added to the org.

    If the user is already in your org, this will return a 400 and a message.

    If the user is already in a different org, this will return a 400 and a message.

    This endpoint requires authentication by an org admin. It adds the specified member to the authenticated user's org."""  # noqa: E501

    return sync_detailed(
        body=body,
        client=client,
    ).parsed


async def asyncio_detailed(
    body: AddOrgMember,
    *,
    client: Client,
) -> Response[Optional[Union[OrgMember, Error]]]:
    kwargs = _get_kwargs(
        body=body,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    body: AddOrgMember,
    *,
    client: Client,
) -> Optional[Union[OrgMember, Error]]:
    """If the user exists, this will add them to your org. If they do not exist, this will create a new user and add them to your org.

    In both cases the user gets an email that they have been added to the org.

    If the user is already in your org, this will return a 400 and a message.

    If the user is already in a different org, this will return a 400 and a message.

    This endpoint requires authentication by an org admin. It adds the specified member to the authenticated user's org."""  # noqa: E501

    return (
        await asyncio_detailed(
            body=body,
            client=client,
        )
    ).parsed



from typing import Optional

from pydantic import BaseModel, ConfigDict


class Solid3dGetCommonEdge(BaseModel):
    """The response from the `Solid3DGetCommonEdge` command."""

    edge: Optional[str] = None

    model_config = ConfigDict(protected_namespaces=())



from pydantic import BaseModel, ConfigDict


class Color(BaseModel):
    """An RGBA color"""

    a: float

    b: float

    g: float

    r: float

    model_config = ConfigDict(protected_namespaces=())



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.file_import_format import FileImportFormat
from ...models.file_surface_area import FileSurfaceArea
from ...models.unit_area import UnitArea
from ...types import Response


def _get_kwargs(
    output_unit: UnitArea,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/file/surface-area".format(
        client.base_url,
    )  # noqa: E501

    if output_unit is not None:
        if "?" in url:
            url = url + "&output_unit=" + str(output_unit)
        else:
            url = url + "?output_unit=" + str(output_unit)

    if src_format is not None:
        if "?" in url:
            url = url + "&src_format=" + str(src_format)
        else:
            url = url + "?src_format=" + str(src_format)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "content": body,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[FileSurfaceArea, Error]]:
    if response.status_code == 201:
        response_201 = FileSurfaceArea(**response.json())
        return response_201
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(
    *, response: httpx.Response
) -> Response[Optional[Union[FileSurfaceArea, Error]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    output_unit: UnitArea,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Response[Optional[Union[FileSurfaceArea, Error]]]:
    kwargs = _get_kwargs(
        output_unit=output_unit,
        src_format=src_format,
        body=body,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    output_unit: UnitArea,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[FileSurfaceArea, Error]]:
    """We assume any file given to us has one consistent unit throughout. We also assume the file is at the proper scale.

    This endpoint returns the square measure units.

    In the future, we will use the units inside the file if they are given and do any conversions if necessary for the calculation. But currently, that is not supported.

    Get the surface area of an object in a CAD file. If the file is larger than 25MB, it will be performed asynchronously.

    If the operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint."""  # noqa: E501

    return sync_detailed(
        output_unit=output_unit,
        src_format=src_format,
        body=body,
        client=client,
    ).parsed


async def asyncio_detailed(
    output_unit: UnitArea,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Response[Optional[Union[FileSurfaceArea, Error]]]:
    kwargs = _get_kwargs(
        output_unit=output_unit,
        src_format=src_format,
        body=body,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    output_unit: UnitArea,
    src_format: FileImportFormat,
    body: bytes,
    *,
    client: Client,
) -> Optional[Union[FileSurfaceArea, Error]]:
    """We assume any file given to us has one consistent unit throughout. We also assume the file is at the proper scale.

    This endpoint returns the square measure units.

    In the future, we will use the units inside the file if they are given and do any conversions if necessary for the calculation. But currently, that is not supported.

    Get the surface area of an object in a CAD file. If the file is larger than 25MB, it will be performed asynchronously.

    If the operation is performed asynchronously, the `id` of the operation will be returned. You can use the `id` returned from the request to get status information about the async operation from the `/async/operations/{id}` endpoint."""  # noqa: E501

    return (
        await asyncio_detailed(
            output_unit=output_unit,
            src_format=src_format,
            body=body,
            client=client,
        )
    ).parsed



from enum import Enum


class ModelingAppIndividualSubscriptionTier(str, Enum):
    """The subscription tiers we offer for the Modeling App to individuals."""  # noqa: E501

    """# The free tier. """  # noqa: E501
    FREE = "free"
    """# The pro tier. """  # noqa: E501
    PRO = "pro"

    def __str__(self) -> str:
        return str(self.value)
